@@ -76,7 +76,7 @@ impl std::fmt::Debug for UniqueIdRef {
 // TODO: We should probably refactor to use `ParseContext`
 //  everywhere we currently use these two things together (particularly `earley.rs`).
 custom_derive! {
-    #[derive(Clone, Reifiable, Debug)]
+    #[derive(Clone, Debug)]
     pub struct ParseContext {
         pub grammar: SynEnv,
         pub type_ctxt: LazyWalkReses<crate::ty::SynthTy>,
@@ -84,6 +84,19 @@ custom_derive! {
     }
 }
 
+impl crate::runtime::reify::Reifiable for ParseContext {
+    fn ty_name() -> Name { n("Language") }
+
+    fn reify(&self) -> crate::runtime::eval::Value {
+        crate::runtime::eval::Value::Language(Box::new(self.clone()))
+    }
+
+    fn reflect(v: &crate::runtime::eval::Value) -> ParseContext {
+        extract!((v) crate::runtime::eval::Value::Language = (ref lang)
+            => (**lang).clone())
+    }
+}
+
 impl PartialEq for ParseContext {
     fn eq(&self, other: &ParseContext) -> bool {
         self as *const ParseContext == other as *const ParseContext
@@ -670,11 +683,10 @@ impl Item {
                 log!("===Ambiguity===\n");
                 // Show both parses...
                 *self.local_parse.borrow_mut() = l;
-                let l_res = self.c_parse(chart, done_tok);
+                let l_res = self.c_parse(chart, done_tok).unwrap();
                 *self.local_parse.borrow_mut() = r;
-                let r_res = self.c_parse(chart, done_tok);
-
-                panic!("Ambiguity! \n{:#?}\n{:#?}\n", l_res, r_res)
+                let r_res = self.c_parse(chart, done_tok).unwrap();
+                panic!("Ambiguity! \n=L=>{}\n=R=>{}\n", l_res, r_res)
             }
             _ => icp!("tried to parse unjustified item: {:#?} ", self),
         };
