@@ -32,6 +32,12 @@ pub fn string_operations() -> Assoc<Name, TypedValue> {
         (Text(s)) =>
             Sequence(s.chars().map(|c: char| Rc::new(Int(BigInt::from(c as u32)))).collect())
     },
+    "anything_to_string" => tyf!{ { "Type" "forall_type" :
+        "param" => ["T"],
+        "body" => (import [* [forall "param"]] { "Type" "fn" :
+            "param" => [(vr "T")],
+            "ret" => {"Type" "String" :}})},
+        (anything) => Text(format!("{}", anything)) },
     "ident_to_string" => tyf! {
         {"Type" "fn" :
             "param" => [{"Type" "Ident" :}],
@@ -50,6 +56,12 @@ pub fn string_operations() -> Assoc<Name, TypedValue> {
             "ret" => {"Type" "String" :}
         },
         (Text(body), Text(old), Text(new)) => Text(body.replace(&old, &new))},
+    "contains?" => tyf! {
+        {"Type" "fn" :
+            "param" => [{"Type" "String" :}, {"Type" "String" :}],
+            "ret" => (vr "Bool")
+        },
+        (Text(lhs), Text(rhs)) => val!(b lhs.contains(&rhs))},
     "join" => tyf! {
         {"Type" "fn" :
             "param" => [{"Type" "type_apply" : "type_rator" => (vr "Sequence"),
@@ -112,7 +124,21 @@ pub fn string_operations() -> Assoc<Name, TypedValue> {
                 .expect("process failure")
                 .stdout).unwrap().to_string())
         }
-    })
+    },
+    "env_var" => tyf! {
+        {"Type" "fn" :
+            "param" => [{"Type" "String" :}],
+            "ret" => {"Type" "type_apply" :
+               "type_rator" => (vr "Option"),
+                "arg" => [{"Type" "String" :}]}},
+        (Text(env_var)) => {
+            match std::env::var(&env_var) {
+                Ok(contents) => val!(enum "Some", (s contents)),
+                Err(_) => val!(enum "None", )
+            }
+        }
+    }
+)
 }
 pub fn sequence_operations() -> Assoc<Name, TypedValue> {
     assoc_n!(
@@ -318,6 +344,18 @@ fn make_core_typed_values() -> Assoc<Name, TypedValue> {
         tyf!( {"Type" "fn" : "param" => [ {"Type" "Int" :}, {"Type" "Int" :} ],
                              "ret" => (vr "Bool")},
               ( Int(a), Int(b) ) => val!(b a == b) ),
+        "and" =>
+        tyf!( {"Type" "fn" : "param" => [ (vr "Bool"), (vr "Bool") ],
+                            "ret" => (vr "Bool")},
+            ( Enum(lhs, _) , Enum(rhs, _)) => val!(b (lhs == n("True")) && (rhs == n("True"))) ),
+        "or" =>
+        tyf!( {"Type" "fn" : "param" => [ (vr "Bool"), (vr "Bool") ],
+                            "ret" => (vr "Bool")},
+            ( Enum(lhs, _) , Enum(rhs, _)) => val!(b (lhs == n("True")) || (rhs == n("True"))) ),
+        "not" =>
+        tyf!( {"Type" "fn" : "param" => [ (vr "Bool") ],
+                            "ret" => (vr "Bool")},
+            ( Enum(arg, _) ) => val!(b !(arg == n("True"))) ),
         "zero" => tf!( "Int", val!(i 0) ),
         "one" => tf!( "Int", val!(i 1) ),
         "two" => tf!( "Int", val!(i 2) ),
@@ -353,6 +391,13 @@ pub fn core_types() -> Assoc<Name, Ast> {
         .set(
             n("Bool"),
             ast!({"Type" "enum" : "name" => [@"c" "True", "False"], "component" => [@"c" [], []]}))
+        .set(
+            n("Option"),
+            ast!({ "Type" "forall_type" :
+                "param" => ["T"],
+                "body" => (import [* [forall "param"]]
+                    {"Type" "enum" : "name" => [@"c" "Some", "None"],
+                                     "component" => [@"c" [], [(vr "T")]]})}))
         .set(
             n("Unit"),
             ast!({"Type" "tuple" : "component" => []}))
